package com.mydomain.jsf.components;

import java.io.IOException;
import java.io.Writer;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import javax.el.ValueExpression;
import javax.faces.component.UIOutput;
import javax.faces.context.FacesContext;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import mondrian.olap.Axis;
import mondrian.olap.Cell;
import mondrian.olap.Member;
import mondrian.olap.Position;
import mondrian.olap.Result;

public class MondrianChartComponent extends UIOutput {
	
	private String name;
	private String query;
	private boolean debug;
	private String title;
	private int width;
	private int height;

	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public int getWidth() {
		return width;
	}

	public void setWidth(int width) {
		this.width = width;
	}

	public int getHeight() {
		return height;
	}

	public void setHeight(int height) {
		this.height = height;
	}

	public boolean isDebug() {
		return debug;
	}

	public void setDebug(boolean debug) {
		this.debug = debug;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getQuery() {
		return query;
	}

	public void setQuery(String query) {
		this.query = query;
	}

	@Override
	public void restoreState(FacesContext context, Object state) {
		// TODO Auto-generated method stub
		super.restoreState(context, state);
	}

	@Override
	public Object saveState(FacesContext context) {
		// TODO Auto-generated method stub
		return super.saveState(context);
	}

	@Override
	public void encodeBegin(FacesContext ctx) throws IOException {
		ResultCache rc = ResultCache.getInstance((HttpSession)ctx.getExternalContext().getSession(true),
				(ServletContext) ctx.getExternalContext().getContext(), name);
		if ( query != null && rc.getQuery() == null ) {
			rc.parse(query);
		}

		Result res = rc.getResult();
		
		HttpServletRequest req = (HttpServletRequest) ctx.getExternalContext().getRequest();
		
		Axis[] axes = res.getAxes();
		int dim = 0;
		for (Axis axis : axes) {
			StringBuilder labels = new StringBuilder();
			StringBuilder values = new StringBuilder();
			int k = 0;
			for (Iterator<Position> j = axis.getPositions().iterator(); j.hasNext(); ) {
				Position pos = j.next();
				for (Iterator<Member> i = pos.iterator(); i.hasNext();) {
					Member member = i.next();
					if( member.isMeasure() || member.isAll() ) {
						labels.append(" "+member);
					}
				}
				Cell cell = res.getCell(new int[]{k++,0});
				values.append(cell.getValue().toString());
				if(j.hasNext()) {
					labels.append(",");
					values.append(",");
				}
			} // pos
			req.setAttribute(name+"_axis"+dim, labels.toString());
			req.setAttribute(name+"_values_"+dim, values.toString());
			dim++;
		}
		
		Writer w = ctx.getResponseWriter();
		
		//w.append("<script type=\"text/javascript\" src=\"js/swfobject.js\"></script>");
		w.append("<div id=\""+name+"\"></div>\n");
		w.append("<script type=\"text/javascript\">\n"+
				"var so_"+name+" = new SWFObject(\"open-flash-chart-1.swf\", \""+name+"\", \""+width+
				"\", \""+height+"\", \"9\", \"#FFFFFF\");\n" );
		
		w.append("so_"+name+".addVariable(\"variables\",\"true\");\n");

		Set<Entry<String, Object>> entrySet = getAttributes().entrySet();
		for (Entry<String, Object> entry : entrySet) {
			if( entry.getKey().startsWith("ofc_")) {
				w.append("so_"+name+".addVariable(\""+entry.getKey().substring(4)+
						"\",\""+entry.getValue().toString()+"\");\n");
			}
		}
		Set<Entry<String, ValueExpression>> bindings = this.bindings.entrySet();
		for (Entry<String, ValueExpression> binding : bindings) {
			if( binding.getKey().startsWith("ofc_")) {
				String expr = binding.getValue().getValue(ctx.getELContext()).toString();
				w.append("so_"+name+".addVariable(\""+binding.getKey().substring(4)+
						"\",\""+expr+"\");\n");
			}
		}
		// json darstellung fr legende, achsen beschriftung, Werte
		w.append("so_"+name+".write(\""+name+"\");\n");
		w.append("</script>\n");
	}

	@Override
	public void encodeChildren(FacesContext arg0) throws IOException {
		super.encodeChildren(arg0);
	}

	@Override
	public void encodeEnd(FacesContext ctx) throws IOException {
//		Writer w = ctx.getResponseWriter();
//		w.append("';}\n</script><div id=\""+name+"\"></div>");
	}
	
	@Override
	public boolean isRendered() {
		return true;
	}


}



