package com.mydomain.jsf.components;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.io.Writer;

import javax.faces.component.UIOutput;
import javax.faces.context.FacesContext;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpSession;
import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import mondrian.olap.Result;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class MondrianComponent extends UIOutput {
	
	private String name;
	private String query;
	private boolean debug;
	private String xsltURI;

	public boolean isDebug() {
		return debug;
	}

	public void setDebug(boolean debug) {
		this.debug = debug;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getQuery() {
		return query;
	}

	public void setQuery(String query) {
		this.query = query;
	}

	@Override
	public void restoreState(FacesContext context, Object state) {
		// TODO Auto-generated method stub
		super.restoreState(context, state);
	}

	@Override
	public Object saveState(FacesContext context) {
		// TODO Auto-generated method stub
		return super.saveState(context);
	}

	@Override
	public void encodeBegin(FacesContext ctx) throws IOException {
		ResultCache rc = ResultCache.getInstance((HttpSession)ctx.getExternalContext().getSession(true),
				(ServletContext) ctx.getExternalContext().getContext(), name);
		if ( query != null && rc.getQuery() == null ) {
			rc.parse(query);
		}
		super.encodeBegin(ctx);
	}

	@Override
	public void encodeChildren(FacesContext arg0) throws IOException {
		super.encodeChildren(arg0);
	}

	@Override
	public void encodeEnd(FacesContext ctx) throws IOException {
		ServletContext sctx = (ServletContext) ctx.getExternalContext().getContext();
		ResultCache rc = ResultCache.getInstance(
				(HttpSession)ctx.getExternalContext().getSession(true),
				sctx, name);

		Document d = rc.getDOM();
		Result res = rc.getResult();
		Writer w = ctx.getResponseWriter();
		ByteArrayOutputStream result = new ByteArrayOutputStream();
		StreamResult sr = null;
		if( debug ) {
			sr = new StreamResult(result);
		} else {
			sr = new StreamResult(w);
		}
		String error = null;
		TransformerFactory tf = TransformerFactory.newInstance();
        InputStream input = sctx.getResourceAsStream(xsltURI);
        if( input == null ) {
        	error = "xsltURI not found: " + xsltURI;
        }
        try {
            Templates templates = tf.newTemplates(new StreamSource(input));
            Transformer transformer = templates.newTransformer();
			transformer.transform(new DOMSource(d), sr);
		} catch (TransformerException e) {
			e.printStackTrace();
		}
		if( error != null ) {
			w.append("<pre>"+escapeXML(error)+"</pre>");
		}
		if(debug) {
			w.append("<pre>"+escapeXML(result.toString("UTF-8"))+"</pre>");
		}
		

	}

	@Override
	public boolean isRendered() {
		return true;
	}

		  static String escapeXML(String s) {
		    StringBuffer str = new StringBuffer();
		    int len = (s != null) ? s.length() : 0;
		    for (int i=0; i<len; i++) {
		       char ch = s.charAt(i);
		       switch (ch) {
		       case '<': str.append("&lt;"); break;
		       case '>': str.append("&gt;"); break;
		       case '&': str.append("&amp;"); break;
		       case '"': str.append("&quot;"); break;
		       case '\'': str.append("&apos;"); break;
		       default: str.append(ch);
		     }
		    }
		    return str.toString();
		  }

		public String getXsltURI() {
			return xsltURI;
		}

		public void setXsltURI(String xsltURI) {
			this.xsltURI = xsltURI;
		}
		}



